import { useRef, useState } from "preact/hooks";
import { useMutation, useQueryClient } from "react-query";
import { patchGuildReminder } from "../../../api";
import { useParams } from "wouter";
import { ICON_FLASH_TIME } from "../../../consts";
import { DeleteButton } from "./DeleteButton";
import { useReminder } from "../ReminderContext";
import { useFlash } from "../../App/FlashContext";

export const EditButtonRow = () => {
    const { guild } = useParams();
    const [reminder, setReminder] = useReminder();

    const [recentlySaved, setRecentlySaved] = useState(false);
    const queryClient = useQueryClient();

    const iconFlashTimeout = useRef(0);

    const flash = useFlash();
    const mutation = useMutation({
        ...patchGuildReminder(guild),
        onSuccess: (response) => {
            queryClient.invalidateQueries({
                queryKey: ["GUILD_REMINDERS", guild],
            });

            if (iconFlashTimeout.current !== null) {
                clearTimeout(iconFlashTimeout.current);
            }

            if (response.data.errors.length > 0) {
                setRecentlySaved(false);

                for (const error of response.data.errors) {
                    flash({ message: error, type: "error" });
                }
            } else {
                setRecentlySaved(true);

                iconFlashTimeout.current = setTimeout(() => {
                    setRecentlySaved(false);
                }, ICON_FLASH_TIME);

                setReminder(response.data.reminder);
            }
        },
    });

    return (
        <div class="button-row-edit">
            <button
                class="button is-success save-btn"
                onClick={() => {
                    mutation.mutate(reminder);
                }}
                disabled={mutation.isLoading}
            >
                <span>Save</span>{" "}
                {mutation.isLoading ? (
                    <span class="icon">
                        <i class="fas fa-spin fa-cog"></i>
                    </span>
                ) : recentlySaved ? (
                    <span class="icon">
                        <i class="fas fa-check"></i>
                    </span>
                ) : (
                    <span class="icon">
                        <i class="fas fa-save"></i>
                    </span>
                )}
            </button>
            <button
                class="button is-warning"
                onClick={() => {
                    mutation.mutate({
                        ...reminder,
                        enabled: !reminder.enabled,
                    });
                }}
                disabled={mutation.isLoading}
            >
                {reminder.enabled ? "Disable" : "Enable"}
            </button>
            <DeleteButton />
        </div>
    );
};
